@@ -24,9 +24,7 @@
  *	Peter Hutterer (peter.hutterer@redhat.com)
  */
 
-#ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif
 
 #ifndef _LIBWACOMINT_H_
 #define _LIBWACOMINT_H_
@@ -35,59 +33,86 @@
 #include <stdint.h>
 #include <glib.h>
 
+#define LIBWACOM_EXPORT __attribute__ ((visibility("default")))
+
 #define DBG(...) \
 	printf(__VA_ARGS__)
 
 #define GENERIC_DEVICE_MATCH "generic"
-#define WACOM_DEVICE_INTEGRATED_UNSET (WACOM_DEVICE_INTEGRATED_NONE - 1)
+#define WACOM_DEVICE_INTEGRATED_UNSET (WACOM_DEVICE_INTEGRATED_NONE - 1U)
 
 enum WacomFeature {
 	FEATURE_STYLUS		= (1 << 0),
 	FEATURE_TOUCH		= (1 << 1),
-	FEATURE_RING		= (1 << 2),
-	FEATURE_RING2		= (1 << 3),
 	FEATURE_REVERSIBLE	= (1 << 4),
 	FEATURE_TOUCHSWITCH	= (1 << 5)
 };
 
-/* WARNING: When adding new members to this struct
- * make sure to update libwacom_copy_match() ! */
-struct _WacomMatch {
-	char *match;
+struct _WacomBuilder {
+	char *device_name;
+	char *match_name;
+	char *uniq;
 	WacomBusType bus;
 	uint32_t vendor_id;
 	uint32_t product_id;
 };
 
+/* WARNING: When adding new members to this struct
+ * make sure to update libwacom_copy_match() ! */
+struct _WacomMatch {
+	gint refcnt;
+	char *match;
+	char *name;
+	char *uniq;
+	WacomBusType bus;
+	uint32_t vendor_id;
+	uint32_t product_id;
+};
+
+/* Used in the device->buttons hashtable */
+typedef struct _WacomButton {
+	WacomButtonFlags flags;
+	int code;
+} WacomButton;
+
+typedef struct _WacomKeycode {
+	unsigned int type;
+	unsigned int code;
+} WacomKeycode;
+
 /* WARNING: When adding new members to this struct
  * make sure to update libwacom_copy() and
  * libwacom_print_device_description() ! */
 struct _WacomDevice {
 	char *name;
+	char *model_name;
 	int width;
 	int height;
 
-	int match;	/* used match or first match by default */
-	WacomMatch **matches; /* NULL-terminated */
-	int nmatches; /* not counting NULL-terminated element */
+	WacomMatch *match;	/* used match or first match by default */
+	GArray *matches;
+
+	WacomMatch *paired;
 
 	WacomClass cls;
 	int num_strips;
+	int num_rings;
+	int num_dials;
 	uint32_t features;
 	uint32_t integration_flags;
 
 	int strips_num_modes;
+	int dial_num_modes;
+	int dial2_num_modes;
 	int ring_num_modes;
 	int ring2_num_modes;
 
-	gsize num_styli;
-	int *supported_styli;
+	GArray *styli;
+	GHashTable *buttons; /* 'A' : WacomButton */
+	WacomKeycode keycodes[32];
+	size_t num_keycodes;
 
-	int num_buttons;
-	WacomButtonFlags *buttons;
-
-	int num_leds;
-	WacomStatusLEDs *status_leds;
+	GArray *status_leds;
 
 	char *layout;
 
@@ -95,13 +120,18 @@ struct _WacomDevice {
 };
 
 struct _WacomStylus {
+	gint refcnt;
 	int id;
 	char *name;
+	char *group;
 	int num_buttons;
 	gboolean has_eraser;
-	gboolean is_eraser;
+	GArray *paired_ids;
+	WacomEraserType eraser_type;
 	gboolean has_lens;
+	gboolean has_wheel;
 	WacomStylusType type;
+	WacomAxisTypeFlags axes;
 };
 
 struct _WacomDeviceDatabase {
@@ -114,15 +144,24 @@ struct _WacomError {
 	char *msg;
 };
 
-/* INTERNAL */
+WacomDevice* libwacom_ref(WacomDevice *device);
+WacomDevice* libwacom_unref(WacomDevice *device);
+WacomStylus* libwacom_stylus_ref(WacomStylus *stylus);
+WacomStylus* libwacom_stylus_unref(WacomStylus *stylus);
+WacomMatch* libwacom_match_ref(WacomMatch *match);
+WacomMatch* libwacom_match_unref(WacomMatch *match);
+
 void libwacom_error_set(WacomError *error, enum WacomErrorCode code, const char *msg, ...);
-void libwacom_stylus_destroy(WacomStylus *stylus);
-void libwacom_update_match(WacomDevice *device, WacomBusType bus, int vendor_id, int product_id);
+void libwacom_add_match(WacomDevice *device, WacomMatch *newmatch);
+void libwacom_set_default_match(WacomDevice *device, WacomMatch *newmatch);
+void libwacom_remove_match(WacomDevice *device, WacomMatch *newmatch);
+WacomMatch* libwacom_match_new(const char *name, const char *uniq,
+			       WacomBusType bus,
+			       int vendor_id, int product_id);
 
 WacomBusType  bus_from_str (const char *str);
 const char   *bus_to_str   (WacomBusType bus);
-char *make_match_string(WacomBusType bus, int vendor_id, int product_id);
-
+char *make_match_string(const char *name, const char *uniq, WacomBusType bus, int vendor_id, int product_id);
 
 #endif /* _LIBWACOMINT_H_ */
 
